@@ -17,7 +17,6 @@
  */
 package org.apache.hadoop.hdfs.server.namenode.ha;
 
-import static org.apache.hadoop.hdfs.server.namenode.NameNodeAdapter.*;
 import static org.junit.Assert.assertEquals;
 import static org.junit.Assert.assertNotNull;
 import static org.junit.Assert.assertNull;
@@ -41,9 +40,8 @@ import org.apache.hadoop.hdfs.qjournal.MiniQJMHACluster;
 import org.apache.hadoop.hdfs.qjournal.server.JournalTestUtil;
 import org.apache.hadoop.hdfs.server.namenode.NNStorage;
 import org.apache.hadoop.hdfs.server.namenode.NameNode;
-import org.apache.hadoop.hdfs.server.namenode.NameNodeAdapter;
 import org.apache.hadoop.test.GenericTestUtils;
-import static org.apache.hadoop.hdfs.qjournal.client.QuorumJournalManager.QJM_RPC_MAX_TXNS_KEY;
+import static org.apache.hadoop.hdfs.server.namenode.NameNodeAdapter.getFileInfo;
 
 import org.junit.After;
 import org.junit.Before;
@@ -74,8 +72,6 @@ public class TestStandbyInProgressTail {
     conf.setBoolean(DFSConfigKeys.DFS_HA_TAILEDITS_INPROGRESS_KEY, true);
     conf.setInt(DFSConfigKeys.DFS_QJOURNAL_SELECT_INPUT_STREAMS_TIMEOUT_KEY,
         500);
-    // Set very samll limit of transactions per a journal rpc call
-    conf.setInt(QJM_RPC_MAX_TXNS_KEY, 3);
     HAUtil.setAllowStandbyReads(conf, true);
     qjmhaCluster = new MiniQJMHACluster.Builder(conf).build();
     cluster = qjmhaCluster.getDfsCluster();
@@ -130,7 +126,7 @@ public class TestStandbyInProgressTail {
 
       // StandbyNameNode should not finish tailing in-progress logs
       assertNull(getFileInfo(cluster.getNameNode(1),
-              "/test", true));
+              "/test", true, false, false));
 
       // Restarting the standby should not finalize any edits files
       // in the shared directory when it starts up!
@@ -146,7 +142,7 @@ public class TestStandbyInProgressTail {
       // either replay starting in the middle of the segment (not allowed)
       // or double-replay the edits (incorrect).
       assertNull(getFileInfo(cluster.getNameNode(1),
-              "/test", true));
+              "/test", true, false, false));
 
       cluster.getNameNode(0).getRpcServer().mkdirs("/test2",
               FsPermission.createImmutable((short) 0755), true);
@@ -159,9 +155,9 @@ public class TestStandbyInProgressTail {
       // NN1 should have both the edits that came before its restart,
       // and the edits that came after its restart.
       assertNotNull(getFileInfo(cluster.getNameNode(1),
-              "/test", true));
+              "/test", true, false, false));
       assertNotNull(getFileInfo(cluster.getNameNode(1),
-              "/test2", true));
+              "/test2", true, false, false));
     } finally {
       if (qjmhaCluster != null) {
         qjmhaCluster.shutdown();
@@ -203,7 +199,7 @@ public class TestStandbyInProgressTail {
 
     // Because we're using in-progress tailer, this should not be null
     assertNotNull(getFileInfo(cluster.getNameNode(1),
-            "/test", true));
+            "/test", true, false, false));
 
     cluster.getNameNode(0).getRpcServer().mkdirs("/test2",
             FsPermission.createImmutable((short) 0755), true);
@@ -216,9 +212,9 @@ public class TestStandbyInProgressTail {
     // NN1 should have both the edits that came before its restart,
     // and the edits that came after its restart.
     assertNotNull(getFileInfo(cluster.getNameNode(1),
-            "/test", true));
+            "/test", true, false, false));
     assertNotNull(getFileInfo(cluster.getNameNode(1),
-            "/test2", true));
+            "/test2", true, false, false));
   }
 
   @Test
@@ -234,7 +230,6 @@ public class TestStandbyInProgressTail {
 
     cluster.getNameNode(0).getRpcServer().mkdirs("/test",
             FsPermission.createImmutable((short) 0755), true);
-    nn1.getNamesystem().getEditLogTailer().doTailEdits();
 
     // StandbyNameNode should tail the in-progress edit
     waitForFileInfo(nn1, "/test");
@@ -250,7 +245,6 @@ public class TestStandbyInProgressTail {
     // Create a new in-progress edit and let SBNN do the tail
     cluster.getNameNode(0).getRpcServer().mkdirs("/test3",
             FsPermission.createImmutable((short) 0755), true);
-    nn1.getNamesystem().getEditLogTailer().doTailEdits();
 
     // StandbyNameNode should tail the finalized edit and the new in-progress
     waitForFileInfo(nn1, "/test", "/test2", "/test3");
@@ -276,9 +270,9 @@ public class TestStandbyInProgressTail {
     cluster.getNameNode(0).getRpcServer().mkdirs("/test3",
             FsPermission.createImmutable((short) 0755), true);
 
-    assertNull(getFileInfo(nn1, "/test", true));
-    assertNull(getFileInfo(nn1, "/test2", true));
-    assertNull(getFileInfo(nn1, "/test3", true));
+    assertNull(getFileInfo(nn1, "/test", true, false, false));
+    assertNull(getFileInfo(nn1, "/test2", true, false, false));
+    assertNull(getFileInfo(nn1, "/test3", true, false, false));
 
     // StandbyNameNode should tail the finalized edit and the new in-progress
     waitForFileInfo(nn1, "/test", "/test2", "/test3");
@@ -306,6 +300,24 @@ public class TestStandbyInProgressTail {
     waitForFileInfo(nn1, "/test", "/test2", "/test3");
   }
 
+  @Test
+  public void testNonUniformConfig() throws Exception {
+    // Test case where some NNs (in this case the active NN) in the cluster
+    // do not have in-progress tailing enabled.
+    Configuration newConf = cluster.getNameNode(0).getConf();
+    newConf.setBoolean(
+        DFSConfigKeys.DFS_HA_TAILEDITS_INPROGRESS_KEY,
+        false);
+    cluster.restartNameNode(0);
+    cluster.transitionToActive(0);
+
+    cluster.getNameNode(0).getRpcServer().mkdirs("/test",
+        FsPermission.createImmutable((short) 0755), true);
+    cluster.getNameNode(0).getRpcServer().rollEdits();
+
+    waitForFileInfo(nn1, "/test");
+  }
+
   @Test
   public void testEditsServedViaCache() throws Exception {
     cluster.transitionToActive(0);
@@ -331,7 +343,7 @@ public class TestStandbyInProgressTail {
     }
     // Ensure edits were not tailed before the edit files were deleted;
     // quick spot check of a single dir
-    assertNull(getFileInfo(nn1, "/tmp0", false));
+    assertNull(getFileInfo(nn1, "/tmp0", false, false, false));
 
     waitForFileInfo(nn1, "/test", "/test2");
   }
@@ -384,50 +396,6 @@ public class TestStandbyInProgressTail {
     waitForFileInfo(nn1, "/test", "/test2", "/test3");
   }
 
-  /**
-   * Test that Standby Node tails multiple segments while catching up
-   * during the transition to Active.
-   */
-  @Test
-  public void testUndertailingWhileFailover() throws Exception {
-    cluster.transitionToActive(0);
-    cluster.waitActive(0);
-
-    String p = "/testFailoverWhileTailingWithoutCache/";
-    mkdirs(nn0, p + 0, p + 1, p + 2, p + 3, p + 4);
-    nn0.getRpcServer().rollEditLog(); // create segment 1
-
-    mkdirs(nn0, p + 5, p + 6, p + 7, p + 8, p + 9);
-    nn0.getRpcServer().rollEditLog(); // create segment 2
-
-    mkdirs(nn0, p + 10, p + 11, p + 12, p + 13, p + 14);
-    nn0.getRpcServer().rollEditLog(); // create segment 3
-
-    cluster.transitionToStandby(0);
-    cluster.transitionToActive(1);
-    cluster.waitActive(1);
-    waitForFileInfo(nn1, p + 0, p + 1, p + 14);
-  }
-
-  @Test
-  public void testNonUniformConfig() throws Exception {
-    // Test case where some NNs (in this case the active NN) in the cluster
-    // do not have in-progress tailing enabled.
-    Configuration newConf = cluster.getNameNode(0).getConf();
-    newConf.setBoolean(
-        DFSConfigKeys.DFS_HA_TAILEDITS_INPROGRESS_KEY,
-        false);
-    cluster.restartNameNode(0);
-    cluster.transitionToActive(0);
-
-    cluster.getNameNode(0).getRpcServer().mkdirs("/test",
-        FsPermission.createImmutable((short) 0755), true);
-    cluster.getNameNode(0).getRpcServer().rollEdits();
-
-    cluster.getNameNode(1).getNamesystem().getEditLogTailer().doTailEdits();
-    assertNotNull(NameNodeAdapter.getFileInfo(nn1, "/test", true));
-  }
-
   /**
    * Check that no edits files are present in the given storage dirs.
    */
@@ -471,16 +439,16 @@ public class TestStandbyInProgressTail {
    * Wait up to 1 second until the given NameNode is aware of the existing of
    * all of the provided fileNames.
    */
-  private static void waitForFileInfo(final NameNode standbyNN, String... fileNames)
+  private static void waitForFileInfo(NameNode standbyNN, String... fileNames)
       throws Exception {
-    final List<String> remainingFiles = Lists.newArrayList(fileNames);
+    List<String> remainingFiles = Lists.newArrayList(fileNames);
     GenericTestUtils.waitFor(new Supplier<Boolean>() {
       @Override
       public Boolean get() {
         try {
           standbyNN.getNamesystem().getEditLogTailer().doTailEdits();
           for (Iterator<String> it = remainingFiles.iterator(); it.hasNext();) {
-            if (getFileInfo(standbyNN, it.next(), true) == null) {
+            if (getFileInfo(standbyNN, it.next(), true, false, false) == null) {
               return false;
             } else {
               it.remove();
