@@ -18,6 +18,7 @@
 
 package org.apache.hadoop.yarn.server.nodemanager.containermanager.linux.resources.gpu;
 
+import org.apache.commons.io.FileUtils;
 import org.apache.hadoop.conf.Configuration;
 import org.apache.hadoop.util.StringUtils;
 import org.apache.hadoop.yarn.api.records.ApplicationAttemptId;
@@ -40,13 +41,15 @@ import org.apache.hadoop.yarn.server.nodemanager.containermanager.resourceplugin
 import org.apache.hadoop.yarn.server.nodemanager.containermanager.runtime.ContainerRuntimeConstants;
 import org.apache.hadoop.yarn.server.nodemanager.recovery.NMNullStateStoreService;
 import org.apache.hadoop.yarn.server.nodemanager.recovery.NMStateStoreService;
+import org.junit.After;
 import org.apache.hadoop.yarn.util.resource.TestResourceUtils;
 import org.junit.Assert;
 import org.junit.Before;
 import org.junit.Test;
 
+import java.io.File;
+import java.io.FileOutputStream;
 import java.io.IOException;
-import java.io.Serializable;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Collections;
@@ -57,7 +60,6 @@ import java.util.concurrent.ConcurrentHashMap;
 
 import static org.mockito.Matchers.any;
 import static org.mockito.Matchers.anyList;
-import static org.mockito.Matchers.anyListOf;
 import static org.mockito.Matchers.anyString;
 import static org.mockito.Matchers.eq;
 import static org.mockito.Mockito.doThrow;
@@ -73,9 +75,43 @@ public class TestGpuResourceHandler {
   private GpuResourceHandlerImpl gpuResourceHandler;
   private NMStateStoreService mockNMStateStore;
   private ConcurrentHashMap<ContainerId, Container> runningContainersMap;
+  private GpuDiscoverer gpuDiscoverer;
+  private File testDataDirectory;
+
+  public void createTestDataDirectory() throws IOException {
+    String testDirectoryPath = getTestParentDirectory();
+    testDataDirectory = new File(testDirectoryPath);
+    FileUtils.deleteDirectory(testDataDirectory);
+    testDataDirectory.mkdirs();
+  }
+
+  private String getTestParentDirectory() {
+    File f = new File("target/temp/" + TestGpuResourceHandler.class.getName());
+    return f.getAbsolutePath();
+  }
+
+  private void touchFile(File f) throws IOException {
+    new FileOutputStream(f).close();
+  }
+
+  private Configuration createDefaultConfig() throws IOException {
+    Configuration conf = new YarnConfiguration();
+    File fakeBinary = setupFakeGpuDiscoveryBinary();
+    conf.set(YarnConfiguration.NM_GPU_PATH_TO_EXEC,
+        fakeBinary.getAbsolutePath());
+    return conf;
+  }
+
+  private File setupFakeGpuDiscoveryBinary() throws IOException {
+    File fakeBinary = new File(getTestParentDirectory() + "/fake-nvidia-smi");
+    touchFile(fakeBinary);
+    return fakeBinary;
+  }
 
   @Before
-  public void setup() {
+  public void setup() throws IOException {
+    createTestDataDirectory();
+
     TestResourceUtils.addNewTypesToResources(ResourceInformation.GPU_URI);
 
     mockCGroupsHandler = mock(CGroupsHandler.class);
@@ -87,16 +123,22 @@ public class TestGpuResourceHandler {
     runningContainersMap = new ConcurrentHashMap<>();
     when(nmctx.getContainers()).thenReturn(runningContainersMap);
 
+    gpuDiscoverer = new GpuDiscoverer();
     gpuResourceHandler = new GpuResourceHandlerImpl(nmctx, mockCGroupsHandler,
-        mockPrivilegedExecutor);
+        mockPrivilegedExecutor, gpuDiscoverer);
+  }
+
+  @After
+  public void cleanupTestFiles() throws IOException {
+    FileUtils.deleteDirectory(testDataDirectory);
   }
 
   @Test
   public void testBootStrap() throws Exception {
-    Configuration conf = new YarnConfiguration();
+    Configuration conf = createDefaultConfig();
     conf.set(YarnConfiguration.NM_GPU_ALLOWED_DEVICES, "0:0");
 
-    GpuDiscoverer.getInstance().initialize(conf);
+    gpuDiscoverer.initialize(conf);
 
     gpuResourceHandler.bootstrap(conf);
     verify(mockCGroupsHandler, times(1)).initializeCGroupController(
@@ -157,9 +199,9 @@ public class TestGpuResourceHandler {
 
   private void commonTestAllocation(boolean dockerContainerEnabled)
       throws Exception {
-    Configuration conf = new YarnConfiguration();
+    Configuration conf = createDefaultConfig();
     conf.set(YarnConfiguration.NM_GPU_ALLOWED_DEVICES, "0:0,1:1,2:3,3:4");
-    GpuDiscoverer.getInstance().initialize(conf);
+    gpuDiscoverer.initialize(conf);
 
     gpuResourceHandler.bootstrap(conf);
     Assert.assertEquals(4,
@@ -171,8 +213,7 @@ public class TestGpuResourceHandler {
 
     // Only device=4 will be blocked.
     if (dockerContainerEnabled) {
-      verifyDeniedDevices(getContainerId(1),
-          Collections.<GpuDevice>emptyList());
+      verifyDeniedDevices(getContainerId(1), Collections.emptyList());
     } else{
       verifyDeniedDevices(getContainerId(1), Arrays.asList(new GpuDevice(3,4)));
     }
@@ -193,8 +234,7 @@ public class TestGpuResourceHandler {
 
     // devices = 0/1/3 will be blocked
     if (dockerContainerEnabled) {
-      verifyDeniedDevices(getContainerId(3),
-          Collections.<GpuDevice>emptyList());
+      verifyDeniedDevices(getContainerId(3), Collections.emptyList());
     } else {
       verifyDeniedDevices(getContainerId(3), Arrays
           .asList(new GpuDevice(0, 0), new GpuDevice(1, 1),
@@ -207,8 +247,7 @@ public class TestGpuResourceHandler {
         mockContainerWithGpuRequest(4, 0, dockerContainerEnabled));
 
     if (dockerContainerEnabled) {
-      verifyDeniedDevices(getContainerId(4),
-          Collections.<GpuDevice>emptyList());
+      verifyDeniedDevices(getContainerId(4), Collections.emptyList());
     } else{
       // All devices will be blocked
       verifyDeniedDevices(getContainerId(4), Arrays
@@ -249,9 +288,9 @@ public class TestGpuResourceHandler {
   @Test
   public void testAssignedGpuWillBeCleanedupWhenStoreOpFails()
       throws Exception {
-    Configuration conf = new YarnConfiguration();
+    Configuration conf = createDefaultConfig();
     conf.set(YarnConfiguration.NM_GPU_ALLOWED_DEVICES, "0:0,1:1,2:3,3:4");
-    GpuDiscoverer.getInstance().initialize(conf);
+    gpuDiscoverer.initialize(conf);
 
     gpuResourceHandler.bootstrap(conf);
     Assert.assertEquals(4,
@@ -278,9 +317,9 @@ public class TestGpuResourceHandler {
 
   @Test
   public void testAllocationWithoutAllowedGpus() throws Exception {
-    Configuration conf = new YarnConfiguration();
+    Configuration conf = createDefaultConfig();
     conf.set(YarnConfiguration.NM_GPU_ALLOWED_DEVICES, " ");
-    GpuDiscoverer.getInstance().initialize(conf);
+    gpuDiscoverer.initialize(conf);
 
     try {
       gpuResourceHandler.bootstrap(conf);
@@ -291,7 +330,7 @@ public class TestGpuResourceHandler {
 
     /* Start container 1, asks 0 containers */
     gpuResourceHandler.preStart(mockContainerWithGpuRequest(1, 0));
-    verifyDeniedDevices(getContainerId(1), Collections.<GpuDevice>emptyList());
+    verifyDeniedDevices(getContainerId(1), Collections.emptyList());
 
     /* Start container 2, asks 1 containers. Excepted to fail */
     boolean failedToAllocate = false;
@@ -313,9 +352,9 @@ public class TestGpuResourceHandler {
 
   @Test
   public void testAllocationStored() throws Exception {
-    Configuration conf = new YarnConfiguration();
+    Configuration conf = createDefaultConfig();
     conf.set(YarnConfiguration.NM_GPU_ALLOWED_DEVICES, "0:0,1:1,2:3,3:4");
-    GpuDiscoverer.getInstance().initialize(conf);
+    gpuDiscoverer.initialize(conf);
 
     gpuResourceHandler.bootstrap(conf);
     Assert.assertEquals(4,
@@ -327,7 +366,7 @@ public class TestGpuResourceHandler {
 
     verify(mockNMStateStore).storeAssignedResources(container,
         ResourceInformation.GPU_URI, Arrays
-            .<Serializable>asList(new GpuDevice(0, 0), new GpuDevice(1, 1),
+            .asList(new GpuDevice(0, 0), new GpuDevice(1, 1),
                 new GpuDevice(2, 3)));
 
     // Only device=4 will be blocked.
@@ -345,8 +384,7 @@ public class TestGpuResourceHandler {
 
     // Store assigned resource will not be invoked.
     verify(mockNMStateStore, never()).storeAssignedResources(
-        eq(container), eq(ResourceInformation.GPU_URI),
-        anyListOf(Serializable.class));
+        eq(container), eq(ResourceInformation.GPU_URI), anyList());
   }
 
   @Test
@@ -358,11 +396,11 @@ public class TestGpuResourceHandler {
 
     GpuResourceHandlerImpl gpuNULLStateResourceHandler =
         new GpuResourceHandlerImpl(nmnctx, mockCGroupsHandler,
-        mockPrivilegedExecutor);
+        mockPrivilegedExecutor, gpuDiscoverer);
 
-    Configuration conf = new YarnConfiguration();
+    Configuration conf = createDefaultConfig();
     conf.set(YarnConfiguration.NM_GPU_ALLOWED_DEVICES, "0:0,1:1,2:3,3:4");
-    GpuDiscoverer.getInstance().initialize(conf);
+    gpuDiscoverer.initialize(conf);
 
     gpuNULLStateResourceHandler.bootstrap(conf);
     Assert.assertEquals(4,
@@ -374,15 +412,15 @@ public class TestGpuResourceHandler {
 
     verify(nmnctx.getNMStateStore()).storeAssignedResources(container,
         ResourceInformation.GPU_URI, Arrays
-            .<Serializable>asList(new GpuDevice(0, 0), new GpuDevice(1, 1),
+            .asList(new GpuDevice(0, 0), new GpuDevice(1, 1),
                 new GpuDevice(2, 3)));
   }
 
   @Test
   public void testRecoverResourceAllocation() throws Exception {
-    Configuration conf = new YarnConfiguration();
+    Configuration conf = createDefaultConfig();
     conf.set(YarnConfiguration.NM_GPU_ALLOWED_DEVICES, "0:0,1:1,2:3,3:4");
-    GpuDiscoverer.getInstance().initialize(conf);
+    gpuDiscoverer.initialize(conf);
 
     gpuResourceHandler.bootstrap(conf);
     Assert.assertEquals(4,
@@ -393,7 +431,7 @@ public class TestGpuResourceHandler {
     ResourceMappings.AssignedResources ar =
         new ResourceMappings.AssignedResources();
     ar.updateAssignedResources(
-        Arrays.<Serializable>asList(new GpuDevice(1, 1), new GpuDevice(2, 3)));
+        Arrays.asList(new GpuDevice(1, 1), new GpuDevice(2, 3)));
     rmap.addAssignedResources(ResourceInformation.GPU_URI, ar);
     when(nmContainer.getResourceMappings()).thenReturn(rmap);
 
@@ -404,7 +442,7 @@ public class TestGpuResourceHandler {
     gpuResourceHandler.reacquireContainer(getContainerId(1));
 
     Map<GpuDevice, ContainerId> deviceAllocationMapping =
-        gpuResourceHandler.getGpuAllocator().getDeviceAllocationMappingCopy();
+        gpuResourceHandler.getGpuAllocator().getDeviceAllocationMapping();
     Assert.assertEquals(2, deviceAllocationMapping.size());
     Assert.assertTrue(
         deviceAllocationMapping.keySet().contains(new GpuDevice(1, 1)));
@@ -420,7 +458,7 @@ public class TestGpuResourceHandler {
     ar = new ResourceMappings.AssignedResources();
     // id=5 is not in allowed list.
     ar.updateAssignedResources(
-        Arrays.<Serializable>asList(new GpuDevice(3, 4), new GpuDevice(4, 5)));
+        Arrays.asList(new GpuDevice(3, 4), new GpuDevice(4, 5)));
     rmap.addAssignedResources(ResourceInformation.GPU_URI, ar);
     when(nmContainer.getResourceMappings()).thenReturn(rmap);
 
@@ -438,7 +476,7 @@ public class TestGpuResourceHandler {
 
     // Make sure internal state not changed.
     deviceAllocationMapping =
-        gpuResourceHandler.getGpuAllocator().getDeviceAllocationMappingCopy();
+        gpuResourceHandler.getGpuAllocator().getDeviceAllocationMapping();
     Assert.assertEquals(2, deviceAllocationMapping.size());
     Assert.assertTrue(deviceAllocationMapping.keySet()
         .containsAll(Arrays.asList(new GpuDevice(1, 1), new GpuDevice(2, 3))));
@@ -452,7 +490,7 @@ public class TestGpuResourceHandler {
     ar = new ResourceMappings.AssignedResources();
     // id=3 is already assigned
     ar.updateAssignedResources(
-        Arrays.<Serializable>asList(new GpuDevice(3, 4), new GpuDevice(2, 3)));
+        Arrays.asList(new GpuDevice(3, 4), new GpuDevice(2, 3)));
     rmap.addAssignedResources("gpu", ar);
     when(nmContainer.getResourceMappings()).thenReturn(rmap);
 
@@ -470,7 +508,7 @@ public class TestGpuResourceHandler {
 
     // Make sure internal state not changed.
     deviceAllocationMapping =
-        gpuResourceHandler.getGpuAllocator().getDeviceAllocationMappingCopy();
+        gpuResourceHandler.getGpuAllocator().getDeviceAllocationMapping();
     Assert.assertEquals(2, deviceAllocationMapping.size());
     Assert.assertTrue(deviceAllocationMapping.keySet()
         .containsAll(Arrays.asList(new GpuDevice(1, 1), new GpuDevice(2, 3))));
