@@ -18,14 +18,24 @@
 
 package org.apache.hadoop.yarn.server.resourcemanager;
 
-import java.io.BufferedWriter;
+import static org.junit.Assert.assertEquals;
+import static org.junit.Assert.assertTrue;
+import static org.mockito.Matchers.any;
+import static org.mockito.Matchers.anyBoolean;
+import static org.mockito.Matchers.anyListOf;
+import static org.mockito.Matchers.anyString;
+import static org.mockito.Matchers.eq;
+import static org.mockito.Mockito.doReturn;
+import static org.mockito.Mockito.mock;
+import static org.mockito.Mockito.spy;
+import static org.mockito.Mockito.when;
+
 import java.io.File;
 import java.io.FileOutputStream;
-import java.io.FileWriter;
 import java.io.IOException;
-import java.net.InetAddress;
 import java.net.InetSocketAddress;
-import java.net.UnknownHostException;
+import java.security.AccessControlException;
+import java.security.PrivilegedExceptionAction;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Collections;
@@ -39,17 +49,9 @@ import java.util.concurrent.BrokenBarrierException;
 import java.util.concurrent.ConcurrentHashMap;
 import java.util.concurrent.CyclicBarrier;
 
-import com.google.common.collect.ImmutableSet;
-import com.google.common.collect.Sets;
-import org.junit.Assert;
-import org.junit.Assume;
-import org.junit.Test;
-import org.mockito.Matchers;
-
 import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
 import org.apache.hadoop.conf.Configuration;
-import org.apache.hadoop.ipc.Server;
 import org.apache.hadoop.security.UserGroupInformation;
 import org.apache.hadoop.yarn.MockApps;
 import org.apache.hadoop.yarn.api.ApplicationClientProtocol;
@@ -149,6 +151,7 @@ import org.apache.hadoop.yarn.server.resourcemanager.scheduler.capacity.Capacity
 import org.apache.hadoop.yarn.server.resourcemanager.scheduler.fair.FairScheduler;
 import org.apache.hadoop.yarn.server.resourcemanager.security.QueueACLsManager;
 import org.apache.hadoop.yarn.server.resourcemanager.timelineservice.RMTimelineCollectorManager;
+
 import org.apache.hadoop.yarn.server.scheduler.SchedulerRequestKey;
 import org.apache.hadoop.yarn.server.security.ApplicationACLsManager;
 import org.apache.hadoop.yarn.server.utils.BuilderUtils;
@@ -156,18 +159,14 @@ import org.apache.hadoop.yarn.util.Clock;
 import org.apache.hadoop.yarn.util.UTCClock;
 import org.apache.hadoop.yarn.util.resource.ResourceCalculator;
 import org.apache.hadoop.yarn.util.resource.Resources;
+import org.junit.Assert;
+import org.junit.Assume;
+import org.junit.Test;
 
-import static org.junit.Assert.assertEquals;
-import static org.junit.Assert.assertTrue;
-import static org.mockito.Matchers.any;
-import static org.mockito.Matchers.anyBoolean;
-import static org.mockito.Matchers.anyListOf;
-import static org.mockito.Matchers.anyString;
-import static org.mockito.Matchers.eq;
-import static org.mockito.Mockito.doReturn;
-import static org.mockito.Mockito.mock;
-import static org.mockito.Mockito.spy;
-import static org.mockito.Mockito.when;
+import com.google.common.collect.ImmutableSet;
+import com.google.common.collect.Sets;
+import org.mockito.invocation.InvocationOnMock;
+import org.mockito.stubbing.Answer;
 
 public class TestClientRMService {
 
@@ -180,7 +179,6 @@ public class TestClientRMService {
   
   private final static String QUEUE_1 = "Q-1";
   private final static String QUEUE_2 = "Q-2";
-  private final static String APPLICATION_TAG_SC_PREPROCESSOR ="mytag:foo";
 
   @Test
   public void testGetDecommissioningClusterNodes() throws Exception {
@@ -544,7 +542,7 @@ public class TestClientRMService {
     when(
         mockQueueACLsManager.checkAccess(any(UserGroupInformation.class),
             any(QueueACL.class), any(RMApp.class), any(String.class),
-            Matchers.<List<String>>any())).thenReturn(true);
+            any())).thenReturn(true);
     return new ClientRMService(rmContext, yarnScheduler, appManager,
         mockAclsManager, mockQueueACLsManager, null);
   }
@@ -570,51 +568,6 @@ public class TestClientRMService {
     }
   }
 
-  @Test
-  public void testApplicationTagsValidation() throws IOException {
-    YarnConfiguration conf = new YarnConfiguration();
-    int maxtags = 3, appMaxTagLength = 5;
-    conf.setInt(YarnConfiguration.RM_APPLICATION_MAX_TAGS, maxtags);
-    conf.setInt(YarnConfiguration.RM_APPLICATION_MAX_TAG_LENGTH,
-        appMaxTagLength);
-    MockRM rm = new MockRM(conf);
-    rm.init(conf);
-    rm.start();
-
-    ClientRMService rmService = rm.getClientRMService();
-
-    List<String> tags = Arrays.asList("Tag1", "Tag2", "Tag3", "Tag4");
-    validateApplicationTag(rmService, tags,
-        "Too many applicationTags, a maximum of only " + maxtags
-            + " are allowed!");
-
-    tags = Arrays.asList("ApplicationTag1", "ApplicationTag2",
-        "ApplicationTag3");
-    // tags are converted to lowercase in
-    // ApplicationSubmissionContext#setApplicationTags
-    validateApplicationTag(rmService, tags,
-        "Tag applicationtag1 is too long, maximum allowed length of a tag is "
-            + appMaxTagLength);
-
-    tags = Arrays.asList("tãg1", "tag2#");
-    validateApplicationTag(rmService, tags,
-        "A tag can only have ASCII characters! Invalid tag - tãg1");
-    rm.close();
-  }
-
-  private void validateApplicationTag(ClientRMService rmService,
-      List<String> tags, String errorMsg) {
-    SubmitApplicationRequest submitRequest = mockSubmitAppRequest(
-        getApplicationId(101), MockApps.newAppName(), QUEUE_1,
-        new HashSet<String>(tags));
-    try {
-      rmService.submitApplication(submitRequest);
-      Assert.fail();
-    } catch (Exception ex) {
-      Assert.assertTrue(ex.getMessage().contains(errorMsg));
-    }
-  }
-
   @Test
   public void testForceKillApplication() throws Exception {
     YarnConfiguration conf = new YarnConfiguration();
@@ -682,10 +635,261 @@ public class TestClientRMService {
     ApplicationId applicationId =
         BuilderUtils.newApplicationId(System.currentTimeMillis(), 0);
     MoveApplicationAcrossQueuesRequest request =
-        MoveApplicationAcrossQueuesRequest.newInstance(applicationId, "newqueue");
+        MoveApplicationAcrossQueuesRequest.newInstance(applicationId,
+            "newqueue");
     rmService.moveApplicationAcrossQueues(request);
   }
 
+  @Test
+  public void testMoveApplicationSubmitTargetQueue() throws Exception {
+    // move the application as the owner
+    ApplicationId applicationId = getApplicationId(1);
+    UserGroupInformation aclUGI = UserGroupInformation.getCurrentUser();
+    QueueACLsManager queueACLsManager = getQueueAclManager("allowed_queue",
+        QueueACL.SUBMIT_APPLICATIONS, aclUGI);
+    ApplicationACLsManager appAclsManager = getAppAclManager();
+
+    ClientRMService rmService = createClientRMServiceForMoveApplicationRequest(
+        applicationId, aclUGI.getShortUserName(), appAclsManager,
+        queueACLsManager);
+
+    // move as the owner queue in the acl
+    MoveApplicationAcrossQueuesRequest moveAppRequest =
+        MoveApplicationAcrossQueuesRequest.
+        newInstance(applicationId, "allowed_queue");
+    rmService.moveApplicationAcrossQueues(moveAppRequest);
+
+    // move as the owner queue not in the acl
+    moveAppRequest = MoveApplicationAcrossQueuesRequest.newInstance(
+        applicationId, "not_allowed");
+
+    try {
+      rmService.moveApplicationAcrossQueues(moveAppRequest);
+      Assert.fail("The request should fail with an AccessControlException");
+    } catch (YarnException rex) {
+      Assert.assertTrue("AccessControlException is expected",
+          rex.getCause() instanceof AccessControlException);
+    }
+
+    // ACL is owned by "moveuser", move is performed as a different user
+    aclUGI = UserGroupInformation.createUserForTesting("moveuser",
+        new String[]{});
+    queueACLsManager = getQueueAclManager("move_queue",
+        QueueACL.SUBMIT_APPLICATIONS, aclUGI);
+    appAclsManager = getAppAclManager();
+    ClientRMService rmService2 =
+        createClientRMServiceForMoveApplicationRequest(applicationId,
+            aclUGI.getShortUserName(), appAclsManager, queueACLsManager);
+
+    // access to the queue not OK: user not allowed in this queue
+    MoveApplicationAcrossQueuesRequest moveAppRequest2 =
+        MoveApplicationAcrossQueuesRequest.
+            newInstance(applicationId, "move_queue");
+    try {
+      rmService2.moveApplicationAcrossQueues(moveAppRequest2);
+      Assert.fail("The request should fail with an AccessControlException");
+    } catch (YarnException rex) {
+      Assert.assertTrue("AccessControlException is expected",
+          rex.getCause() instanceof AccessControlException);
+    }
+
+    // execute the move as the acl owner
+    // access to the queue OK: user allowed in this queue
+    aclUGI.doAs(new PrivilegedExceptionAction<Object>() {
+      @Override
+      public Object run() throws Exception {
+        return rmService2.moveApplicationAcrossQueues(moveAppRequest2);
+      }
+    });
+  }
+
+  @Test
+  public void testMoveApplicationAdminTargetQueue() throws Exception {
+    ApplicationId applicationId = getApplicationId(1);
+    UserGroupInformation aclUGI = UserGroupInformation.getCurrentUser();
+    QueueACLsManager queueAclsManager = getQueueAclManager("allowed_queue",
+        QueueACL.ADMINISTER_QUEUE, aclUGI);
+    ApplicationACLsManager appAclsManager = getAppAclManager();
+    ClientRMService rmService =
+        createClientRMServiceForMoveApplicationRequest(applicationId,
+            aclUGI.getShortUserName(), appAclsManager, queueAclsManager);
+
+    // user is admin move to queue in acl
+    MoveApplicationAcrossQueuesRequest moveAppRequest =
+        MoveApplicationAcrossQueuesRequest.newInstance(applicationId,
+            "allowed_queue");
+    rmService.moveApplicationAcrossQueues(moveAppRequest);
+
+    // user is admin move to queue not in acl
+    moveAppRequest = MoveApplicationAcrossQueuesRequest.newInstance(
+        applicationId, "not_allowed");
+
+    try {
+      rmService.moveApplicationAcrossQueues(moveAppRequest);
+      Assert.fail("The request should fail with an AccessControlException");
+    } catch (YarnException rex) {
+      Assert.assertTrue("AccessControlException is expected",
+          rex.getCause() instanceof AccessControlException);
+    }
+
+    // ACL is owned by "moveuser", move is performed as a different user
+    aclUGI = UserGroupInformation.createUserForTesting("moveuser",
+        new String[]{});
+    queueAclsManager = getQueueAclManager("move_queue",
+        QueueACL.ADMINISTER_QUEUE, aclUGI);
+    appAclsManager = getAppAclManager();
+    ClientRMService rmService2 =
+        createClientRMServiceForMoveApplicationRequest(applicationId,
+            aclUGI.getShortUserName(), appAclsManager, queueAclsManager);
+
+    // no access to this queue
+    MoveApplicationAcrossQueuesRequest moveAppRequest2 =
+        MoveApplicationAcrossQueuesRequest.
+            newInstance(applicationId, "move_queue");
+
+    try {
+      rmService2.moveApplicationAcrossQueues(moveAppRequest2);
+      Assert.fail("The request should fail with an AccessControlException");
+    } catch (YarnException rex) {
+      Assert.assertTrue("AccessControlException is expected",
+          rex.getCause() instanceof AccessControlException);
+    }
+
+    // execute the move as the acl owner
+    // access to the queue OK: user allowed in this queue
+    aclUGI.doAs(new PrivilegedExceptionAction<Object>() {
+      @Override
+      public Object run() throws Exception {
+        return rmService2.moveApplicationAcrossQueues(moveAppRequest2);
+      }
+    });
+  }
+
+  @Test (expected = YarnException.class)
+  public void testNonExistingQueue() throws Exception {
+    ApplicationId applicationId = getApplicationId(1);
+    UserGroupInformation aclUGI = UserGroupInformation.getCurrentUser();
+    QueueACLsManager queueAclsManager = getQueueAclManager();
+    ApplicationACLsManager appAclsManager = getAppAclManager();
+    ClientRMService rmService =
+        createClientRMServiceForMoveApplicationRequest(applicationId,
+            aclUGI.getShortUserName(), appAclsManager, queueAclsManager);
+
+    MoveApplicationAcrossQueuesRequest moveAppRequest =
+        MoveApplicationAcrossQueuesRequest.newInstance(applicationId,
+            "unknown_queue");
+    rmService.moveApplicationAcrossQueues(moveAppRequest);
+  }
+
+  /**
+   * Create an instance of ClientRMService for testing
+   * moveApplicationAcrossQueues requests.
+   * @param applicationId the application
+   * @return ClientRMService
+   */
+  private ClientRMService createClientRMServiceForMoveApplicationRequest(
+      ApplicationId applicationId, String appOwner,
+      ApplicationACLsManager appAclsManager, QueueACLsManager queueAclsManager)
+      throws IOException {
+    RMApp app = mock(RMApp.class);
+    when(app.getUser()).thenReturn(appOwner);
+    when(app.getState()).thenReturn(RMAppState.RUNNING);
+    ConcurrentHashMap<ApplicationId, RMApp> apps = new ConcurrentHashMap<>();
+    apps.put(applicationId, app);
+
+    RMContext rmContext = mock(RMContext.class);
+    when(rmContext.getRMApps()).thenReturn(apps);
+
+    RMAppManager rmAppManager = mock(RMAppManager.class);
+    return new ClientRMService(rmContext, null, rmAppManager, appAclsManager,
+        queueAclsManager, null);
+  }
+
+  /**
+   * Plain application acl manager that always returns true.
+   * @return ApplicationACLsManager
+   */
+  private ApplicationACLsManager getAppAclManager() {
+    ApplicationACLsManager aclsManager = mock(ApplicationACLsManager.class);
+    when(aclsManager.checkAccess(
+        any(UserGroupInformation.class),
+        any(ApplicationAccessType.class),
+        any(String.class),
+        any(ApplicationId.class))).thenReturn(true);
+    return aclsManager;
+  }
+
+  /**
+   * Generate the Queue acl.
+   * @param allowedQueue the queue to allow the move to
+   * @param queueACL the acl to check: submit app or queue admin
+   * @param aclUser the user to check
+   * @return QueueACLsManager
+   */
+  private QueueACLsManager getQueueAclManager(String allowedQueue,
+      QueueACL queueACL, UserGroupInformation aclUser) throws IOException {
+    // ACL that checks the queue is allowed
+    QueueACLsManager queueACLsManager = mock(QueueACLsManager.class);
+    when(queueACLsManager.checkAccess(
+        any(UserGroupInformation.class),
+        any(QueueACL.class),
+        any(RMApp.class),
+        any(String.class),
+        anyListOf(String.class))).thenAnswer(new Answer<Boolean>() {
+            @Override
+            public Boolean answer(InvocationOnMock invocationOnMock) {
+              final UserGroupInformation user =
+                  (UserGroupInformation) invocationOnMock.getArguments()[0];
+              final QueueACL acl =
+                  (QueueACL) invocationOnMock.getArguments()[1];
+              return (queueACL.equals(acl) &&
+                  aclUser.getShortUserName().equals(user.getShortUserName()));
+            }
+        });
+
+    when(queueACLsManager.checkAccess(
+        any(UserGroupInformation.class),
+        any(QueueACL.class),
+        any(RMApp.class),
+        any(String.class),
+        anyListOf(String.class),
+        any(String.class))).thenAnswer(new Answer<Boolean>() {
+          @Override
+          public Boolean answer(InvocationOnMock invocationOnMock) {
+            final UserGroupInformation user =
+                (UserGroupInformation) invocationOnMock.getArguments()[0];
+            final QueueACL acl = (QueueACL) invocationOnMock.getArguments()[1];
+            final String queue = (String) invocationOnMock.getArguments()[5];
+            return (allowedQueue.equals(queue) && queueACL.equals(acl) &&
+                aclUser.getShortUserName().equals(user.getShortUserName()));
+          }
+        });
+    return queueACLsManager;
+  }
+
+  /**
+   * QueueACLsManager that always returns false when a target queue is passed
+   * in and true for other checks to simulate a missing queue.
+   * @return QueueACLsManager
+   */
+  private QueueACLsManager getQueueAclManager() {
+    QueueACLsManager queueACLsManager = mock(QueueACLsManager.class);
+    when(queueACLsManager.checkAccess(
+        any(UserGroupInformation.class),
+        any(QueueACL.class),
+        any(RMApp.class),
+        any(String.class),
+        anyListOf(String.class),
+        any(String.class))).thenReturn(false);
+    when(queueACLsManager.checkAccess(
+        any(UserGroupInformation.class),
+        any(QueueACL.class),
+        any(RMApp.class),
+        any(String.class),
+        anyListOf(String.class))).thenReturn(true);
+    return queueACLsManager;
+  }
+
   @Test
   public void testGetQueueInfo() throws Exception {
     YarnScheduler yarnScheduler = mock(YarnScheduler.class);
@@ -696,7 +900,7 @@ public class TestClientRMService {
     QueueACLsManager mockQueueACLsManager = mock(QueueACLsManager.class);
     when(mockQueueACLsManager.checkAccess(any(UserGroupInformation.class),
         any(QueueACL.class), any(RMApp.class), any(String.class),
-        Matchers.<List<String>>any()))
+        any()))
         .thenReturn(true);
     when(mockAclsManager.checkAccess(any(UserGroupInformation.class),
         any(ApplicationAccessType.class), anyString(),
@@ -735,7 +939,7 @@ public class TestClientRMService {
         mock(QueueACLsManager.class);
     when(mockQueueACLsManager1.checkAccess(any(UserGroupInformation.class),
         any(QueueACL.class), any(RMApp.class), any(String.class),
-        Matchers.<List<String>>any()))
+        any()))
         .thenReturn(false);
     when(mockAclsManager1.checkAccess(any(UserGroupInformation.class),
         any(ApplicationAccessType.class), anyString(),
@@ -751,178 +955,6 @@ public class TestClientRMService {
     Assert.assertEquals(0, applications1.size());
   }
 
-  @Test (timeout = 30000)
-  @SuppressWarnings ("rawtypes")
-  public void testAppSubmitWithSubmissionPreProcessor() throws Exception {
-    YarnScheduler scheduler = mockYarnScheduler();
-    RMContext rmContext = mock(RMContext.class);
-    mockRMContext(scheduler, rmContext);
-    YarnConfiguration yConf = new YarnConfiguration();
-    yConf.setBoolean(YarnConfiguration.RM_SUBMISSION_PREPROCESSOR_ENABLED,
-        true);
-    yConf.setBoolean(YarnConfiguration.NODE_LABELS_ENABLED, true);
-    // Override the YARN configuration.
-    when(rmContext.getYarnConfiguration()).thenReturn(yConf);
-    RMStateStore stateStore = mock(RMStateStore.class);
-    when(rmContext.getStateStore()).thenReturn(stateStore);
-    RMAppManager appManager = new RMAppManager(rmContext, scheduler,
-        null, mock(ApplicationACLsManager.class), new Configuration());
-    when(rmContext.getDispatcher().getEventHandler()).thenReturn(
-        new EventHandler<Event>() {
-          public void handle(Event event) {}
-        });
-    ApplicationId appId1 = getApplicationId(100);
-    ApplicationACLsManager mockAclsManager = mock(ApplicationACLsManager.class);
-    when(
-        mockAclsManager.checkAccess(UserGroupInformation.getCurrentUser(),
-            ApplicationAccessType.VIEW_APP, null, appId1)).thenReturn(true);
-
-    QueueACLsManager mockQueueACLsManager = mock(QueueACLsManager.class);
-    when(mockQueueACLsManager.checkAccess(any(UserGroupInformation.class),
-        any(QueueACL.class), any(RMApp.class), any(String.class),
-        anyListOf(String.class)))
-        .thenReturn(true);
-
-    ClientRMService rmService =
-        new ClientRMService(rmContext, scheduler, appManager,
-            mockAclsManager, mockQueueACLsManager, null);
-    File rulesFile = File.createTempFile("submission_rules", ".tmp");
-    rulesFile.deleteOnExit();
-    rulesFile.createNewFile();
-
-    yConf.set(YarnConfiguration.RM_SUBMISSION_PREPROCESSOR_FILE_PATH,
-        rulesFile.getAbsolutePath());
-    rmService.serviceInit(yConf);
-    rmService.serviceStart();
-
-    BufferedWriter writer = new BufferedWriter(new FileWriter(rulesFile));
-    writer.write("host.cluster1.com   NL=foo     Q=bar  TA=cluster:cluster1");
-    writer.newLine();
-    writer.write("host.cluster2.com   Q=hello  NL=zuess   TA=cluster:cluster2");
-    writer.newLine();
-    writer.write("host.cluster.*.com   Q=hello  NL=reg   TA=cluster:reg");
-    writer.newLine();
-    writer.write("host.cluster.*.com   Q=hello  NL=reg   TA=cluster:reg");
-    writer.newLine();
-    writer.write("*   TA=cluster:other    Q=default  NL=barfoo");
-    writer.newLine();
-    writer.write("host.testcluster1.com  Q=default");
-    writer.flush();
-    writer.close();
-    rmService.getContextPreProcessor().refresh();
-    setupCurrentCall("host.cluster1.com");
-    SubmitApplicationRequest submitRequest1 = mockSubmitAppRequest(
-        appId1, null, null);
-    try {
-      rmService.submitApplication(submitRequest1);
-    } catch (YarnException e) {
-      Assert.fail("Exception is not expected.");
-    }
-    RMApp app1 = rmContext.getRMApps().get(appId1);
-    Assert.assertNotNull("app doesn't exist", app1);
-    Assert.assertEquals("app name doesn't match",
-        YarnConfiguration.DEFAULT_APPLICATION_NAME, app1.getName());
-    Assert.assertTrue("custom tag not present",
-        app1.getApplicationTags().contains("cluster:cluster1"));
-    Assert.assertEquals("app queue doesn't match", "bar", app1.getQueue());
-    Assert.assertEquals("app node label doesn't match",
-        "foo", app1.getApplicationSubmissionContext().getNodeLabelExpression());
-    setupCurrentCall("host.cluster2.com");
-    ApplicationId appId2 = getApplicationId(101);
-    SubmitApplicationRequest submitRequest2 = mockSubmitAppRequest(
-        appId2, null, null);
-    submitRequest2.getApplicationSubmissionContext().setApplicationType(
-        "matchType");
-    Set<String> aTags = new HashSet<String>();
-    aTags.add(APPLICATION_TAG_SC_PREPROCESSOR);
-    submitRequest2.getApplicationSubmissionContext().setApplicationTags(aTags);
-    try {
-      rmService.submitApplication(submitRequest2);
-    } catch (YarnException e) {
-      Assert.fail("Exception is not expected.");
-    }
-    RMApp app2 = rmContext.getRMApps().get(appId2);
-    Assert.assertNotNull("app doesn't exist", app2);
-    Assert.assertEquals("app name doesn't match",
-        YarnConfiguration.DEFAULT_APPLICATION_NAME, app2.getName());
-    Assert.assertTrue("client tag not present",
-        app2.getApplicationTags().contains(APPLICATION_TAG_SC_PREPROCESSOR));
-    Assert.assertTrue("custom tag not present",
-        app2.getApplicationTags().contains("cluster:cluster2"));
-    Assert.assertEquals("app queue doesn't match", "hello", app2.getQueue());
-    Assert.assertEquals("app node label doesn't match",
-        "zuess",
-        app2.getApplicationSubmissionContext().getNodeLabelExpression());
-    // Test Default commands
-    setupCurrentCall("host2.cluster3.com");
-    ApplicationId appId3 = getApplicationId(102);
-    SubmitApplicationRequest submitRequest3 = mockSubmitAppRequest(
-        appId3, null, null);
-    submitRequest3.getApplicationSubmissionContext().setApplicationType(
-        "matchType");
-    submitRequest3.getApplicationSubmissionContext().setApplicationTags(aTags);
-    try {
-      rmService.submitApplication(submitRequest3);
-    } catch (YarnException e) {
-      Assert.fail("Exception is not expected.");
-    }
-    RMApp app3 = rmContext.getRMApps().get(appId3);
-    Assert.assertNotNull("app doesn't exist", app3);
-    Assert.assertEquals("app name doesn't match",
-        YarnConfiguration.DEFAULT_APPLICATION_NAME, app3.getName());
-    Assert.assertTrue("client tag not present",
-        app3.getApplicationTags().contains(APPLICATION_TAG_SC_PREPROCESSOR));
-    Assert.assertTrue("custom tag not present",
-        app3.getApplicationTags().contains("cluster:other"));
-    Assert.assertEquals("app queue doesn't match", "default", app3.getQueue());
-    Assert.assertEquals("app node label doesn't match",
-        "barfoo",
-        app3.getApplicationSubmissionContext().getNodeLabelExpression());
-    // Test regex
-    setupCurrentCall("host.cluster100.com");
-    ApplicationId appId4 = getApplicationId(103);
-    SubmitApplicationRequest submitRequest4 = mockSubmitAppRequest(
-        appId4, null, null);
-    try {
-      rmService.submitApplication(submitRequest4);
-    } catch (YarnException e) {
-      Assert.fail("Exception is not expected.");
-    }
-    RMApp app4 = rmContext.getRMApps().get(appId4);
-    Assert.assertTrue("custom tag not present",
-        app4.getApplicationTags().contains("cluster:reg"));
-    Assert.assertEquals("app node label doesn't match",
-        "reg", app4.getApplicationSubmissionContext().getNodeLabelExpression());
-    testSubmissionContextWithAbsentTAG(rmService, rmContext);
-    rmService.serviceStop();
-  }
-
-  private void testSubmissionContextWithAbsentTAG(ClientRMService rmService,
-      RMContext rmContext) throws Exception {
-    setupCurrentCall("host.testcluster1.com");
-    ApplicationId appId5 = getApplicationId(104);
-    SubmitApplicationRequest submitRequest5 = mockSubmitAppRequest(
-        appId5, null, null);
-    try {
-      rmService.submitApplication(submitRequest5);
-    } catch (YarnException e) {
-      Assert.fail("Exception is not expected.");
-    }
-    RMApp app5 = rmContext.getRMApps().get(appId5);
-    Assert.assertEquals("custom tag  present",
-        app5.getApplicationTags().size(), 0);
-    Assert.assertNull("app node label present",
-        app5.getApplicationSubmissionContext().getNodeLabelExpression());
-    Assert.assertEquals("Queue name is not present",
-        app5.getQueue(), "default");
-  }
-  private void setupCurrentCall(String hostName) throws UnknownHostException {
-    Server.Call mockCall = mock(Server.Call.class);
-    when(mockCall.getHostInetAddress()).thenReturn(
-                InetAddress.getByAddress(hostName,
-                        new byte[]{123, 123, 123, 123}));
-    Server.getCurCall().set(mockCall);
-  }
   
   @Test (timeout = 30000)
   @SuppressWarnings ("rawtypes")
@@ -951,7 +983,7 @@ public class TestClientRMService {
     QueueACLsManager mockQueueACLsManager = mock(QueueACLsManager.class);
     when(mockQueueACLsManager.checkAccess(any(UserGroupInformation.class),
         any(QueueACL.class), any(RMApp.class), any(String.class),
-        Matchers.<List<String>>any()))
+        any()))
         .thenReturn(true);
     ClientRMService rmService =
         new ClientRMService(rmContext, yarnScheduler, appManager,
@@ -1057,7 +1089,7 @@ public class TestClientRMService {
     QueueACLsManager mockQueueACLsManager = mock(QueueACLsManager.class);
     when(mockQueueACLsManager.checkAccess(any(UserGroupInformation.class),
         any(QueueACL.class), any(RMApp.class), any(String.class),
-        Matchers.<List<String>>any()))
+        any()))
         .thenReturn(true);
     ClientRMService rmService =
         new ClientRMService(rmContext, yarnScheduler, appManager,
@@ -1209,8 +1241,7 @@ public class TestClientRMService {
     final CyclicBarrier startBarrier = new CyclicBarrier(2);
     final CyclicBarrier endBarrier = new CyclicBarrier(2);
 
-    @SuppressWarnings("rawtypes")
-    EventHandler eventHandler = new EventHandler() {
+    EventHandler<Event> eventHandler = new EventHandler<Event>() {
       @Override
       public void handle(Event rawEvent) {
         if (rawEvent instanceof RMAppEvent) {
@@ -1297,7 +1328,8 @@ public class TestClientRMService {
       throws IOException {
     Dispatcher dispatcher = mock(Dispatcher.class);
     when(rmContext.getDispatcher()).thenReturn(dispatcher);
-    EventHandler eventHandler = mock(EventHandler.class);
+    @SuppressWarnings("unchecked")
+    EventHandler<Event> eventHandler = mock(EventHandler.class);
     when(dispatcher.getEventHandler()).thenReturn(eventHandler);
 
     QueueInfo queInfo = recordFactory.newRecordInstance(QueueInfo.class);
@@ -2104,6 +2136,49 @@ public class TestClientRMService {
     new File(excludeFile).delete();
   }
 
+  @Test
+  public void testGetResourceTypesInfoWhenResourceProfileDisabled()
+      throws Exception {
+    YarnConfiguration conf = new YarnConfiguration();
+    MockRM rm = new MockRM(conf) {
+      protected ClientRMService createClientRMService() {
+        return new ClientRMService(this.rmContext, scheduler,
+            this.rmAppManager, this.applicationACLsManager, this.queueACLsManager,
+            this.getRMContext().getRMDelegationTokenSecretManager());
+      }
+    };
+    rm.start();
+
+    YarnRPC rpc = YarnRPC.create(conf);
+    InetSocketAddress rmAddress = rm.getClientRMService().getBindAddress();
+    LOG.info("Connecting to ResourceManager at " + rmAddress);
+    ApplicationClientProtocol client =
+        (ApplicationClientProtocol) rpc
+            .getProxy(ApplicationClientProtocol.class, rmAddress, conf);
+
+    // Make call
+    GetAllResourceTypeInfoRequest request =
+        GetAllResourceTypeInfoRequest.newInstance();
+    GetAllResourceTypeInfoResponse response = client.getResourceTypeInfo(request);
+
+    Assert.assertEquals(2, response.getResourceTypeInfo().size());
+
+    // Check memory
+    Assert.assertEquals(ResourceInformation.MEMORY_MB.getName(),
+        response.getResourceTypeInfo().get(0).getName());
+    Assert.assertEquals(ResourceInformation.MEMORY_MB.getUnits(),
+        response.getResourceTypeInfo().get(0).getDefaultUnit());
+
+    // Check vcores
+    Assert.assertEquals(ResourceInformation.VCORES.getName(),
+        response.getResourceTypeInfo().get(1).getName());
+    Assert.assertEquals(ResourceInformation.VCORES.getUnits(),
+        response.getResourceTypeInfo().get(1).getDefaultUnit());
+
+    rm.stop();
+    rpc.stopProxy(client, conf);
+  }
+
   @Test
   public void testGetApplicationsWithPerUserApps()
       throws IOException, YarnException {
@@ -2116,6 +2191,8 @@ public class TestClientRMService {
     mockRMContext(yarnScheduler, rmContext);
     RMStateStore stateStore = mock(RMStateStore.class);
     when(rmContext.getStateStore()).thenReturn(stateStore);
+    doReturn(mock(RMTimelineCollectorManager.class)).when(rmContext)
+        .getRMTimelineCollectorManager();
 
     RMAppManager appManager = new RMAppManager(rmContext, yarnScheduler, null,
         mock(ApplicationACLsManager.class), new Configuration());
@@ -2169,46 +2246,4 @@ public class TestClientRMService {
         rmService.getApplications(request).getApplicationList().size());
     rmService.setDisplayPerUserApps(false);
   }
-
-  public void testGetResourceTypesInfoWhenResourceProfileDisabled()
-      throws Exception {
-    YarnConfiguration conf = new YarnConfiguration();
-    MockRM rm = new MockRM(conf) {
-      protected ClientRMService createClientRMService() {
-        return new ClientRMService(this.rmContext, scheduler,
-            this.rmAppManager, this.applicationACLsManager, this.queueACLsManager,
-            this.getRMContext().getRMDelegationTokenSecretManager());
-      }
-    };
-    rm.start();
-
-    YarnRPC rpc = YarnRPC.create(conf);
-    InetSocketAddress rmAddress = rm.getClientRMService().getBindAddress();
-    LOG.info("Connecting to ResourceManager at " + rmAddress);
-    ApplicationClientProtocol client =
-        (ApplicationClientProtocol) rpc
-            .getProxy(ApplicationClientProtocol.class, rmAddress, conf);
-
-    // Make call
-    GetAllResourceTypeInfoRequest request =
-        GetAllResourceTypeInfoRequest.newInstance();
-    GetAllResourceTypeInfoResponse response = client.getResourceTypeInfo(request);
-
-    Assert.assertEquals(2, response.getResourceTypeInfo().size());
-
-    // Check memory
-    Assert.assertEquals(ResourceInformation.MEMORY_MB.getName(),
-        response.getResourceTypeInfo().get(0).getName());
-    Assert.assertEquals(ResourceInformation.MEMORY_MB.getUnits(),
-        response.getResourceTypeInfo().get(0).getDefaultUnit());
-
-    // Check vcores
-    Assert.assertEquals(ResourceInformation.VCORES.getName(),
-        response.getResourceTypeInfo().get(1).getName());
-    Assert.assertEquals(ResourceInformation.VCORES.getUnits(),
-        response.getResourceTypeInfo().get(1).getDefaultUnit());
-
-    rm.stop();
-    rpc.stopProxy(client, conf);
-  }
 }
