@@ -474,7 +474,7 @@ public class TestRMAppAttemptTransitions {
 
     assertEquals(expectedState, applicationAttempt.getAppAttemptState());
     verify(scheduler, times(expectedAllocateCount)).allocate(
-        any(ApplicationAttemptId.class), any(List.class), any(List.class),
+        any(ApplicationAttemptId.class), any(List.class), eq(null), any(List.class),
         any(List.class), any(List.class), any(ContainerUpdates.class));
 
     assertEquals(0,applicationAttempt.getJustFinishedContainers().size());
@@ -495,7 +495,7 @@ public class TestRMAppAttemptTransitions {
     // Check events
     verify(applicationMasterLauncher).handle(any(AMLauncherEvent.class));
     verify(scheduler, times(2)).allocate(any(ApplicationAttemptId.class),
-        any(List.class), any(List.class), any(List.class), any(List.class),
+        any(List.class), any(List.class), any(List.class), any(List.class), any(List.class),
         any(ContainerUpdates.class));
     verify(nmTokenManager).clearNodeSetForAttempt(
       applicationAttempt.getAppAttemptId());
@@ -643,14 +643,12 @@ public class TestRMAppAttemptTransitions {
     when(allocation.getContainers()).
         thenReturn(Collections.singletonList(container));
     when(scheduler.allocate(any(ApplicationAttemptId.class), any(List.class),
-        any(List.class), any(List.class), any(List.class),
+        any(List.class), any(List.class), any(List.class), any(List.class),
         any(ContainerUpdates.class))).
     thenReturn(allocation);
     RMContainer rmContainer = mock(RMContainerImpl.class);
     when(scheduler.getRMContainer(container.getId())).
         thenReturn(rmContainer);
-    when(container.getNodeId()).thenReturn(
-        BuilderUtils.newNodeId("localhost", 0));
     
     applicationAttempt.handle(
         new RMAppAttemptEvent(applicationAttempt.getAppAttemptId(),
@@ -988,7 +986,7 @@ public class TestRMAppAttemptTransitions {
   public void testAttemptAddedAtFinalSaving() {
     submitApplicationAttempt();
 
-    // SUBNITED->FINAL_SAVING
+    // SUBMITTED->FINAL_SAVING
     applicationAttempt.handle(new RMAppAttemptEvent(applicationAttempt
                    .getAppAttemptId(), RMAppAttemptEventType.KILL));
     assertEquals(RMAppAttemptState.FINAL_SAVING,
@@ -1001,6 +999,56 @@ public class TestRMAppAttemptTransitions {
                    applicationAttempt.getAppAttemptState());
   }
 
+  @Test(timeout = 10000)
+  public void testAttemptRegisteredAtFailed() {
+    Container amContainer = allocateApplicationAttempt();
+    launchApplicationAttempt(amContainer);
+
+    //send CONTAINER_FINISHED event
+    NodeId anyNodeId = NodeId.newInstance("host", 1234);
+    applicationAttempt.handle(new RMAppAttemptContainerFinishedEvent(
+        applicationAttempt.getAppAttemptId(), BuilderUtils.newContainerStatus(
+        amContainer.getId(), ContainerState.COMPLETE, "", 0,
+        amContainer.getResource()), anyNodeId));
+    assertEquals(RMAppAttemptState.FINAL_SAVING,
+        applicationAttempt.getAppAttemptState());
+
+    sendAttemptUpdateSavedEvent(applicationAttempt);
+    assertEquals(RMAppAttemptState.FAILED,
+        applicationAttempt.getAppAttemptState());
+
+    //send REGISTERED event
+    applicationAttempt.handle(new RMAppAttemptEvent(applicationAttempt
+        .getAppAttemptId(), RMAppAttemptEventType.REGISTERED));
+
+    assertEquals(RMAppAttemptState.FAILED,
+        applicationAttempt.getAppAttemptState());
+  }
+
+  @Test
+  public void testAttemptLaunchFailedAtFailed() {
+    Container amContainer = allocateApplicationAttempt();
+    launchApplicationAttempt(amContainer);
+    //send CONTAINER_FINISHED event
+    NodeId anyNodeId = NodeId.newInstance("host", 1234);
+    applicationAttempt.handle(new RMAppAttemptContainerFinishedEvent(
+        applicationAttempt.getAppAttemptId(), BuilderUtils.newContainerStatus(
+        amContainer.getId(), ContainerState.COMPLETE, "", 0,
+        amContainer.getResource()), anyNodeId));
+    assertEquals(RMAppAttemptState.FINAL_SAVING,
+        applicationAttempt.getAppAttemptState());
+    sendAttemptUpdateSavedEvent(applicationAttempt);
+    assertEquals(RMAppAttemptState.FAILED,
+        applicationAttempt.getAppAttemptState());
+
+    //send LAUNCH_FAILED event
+    applicationAttempt.handle(new RMAppAttemptEvent(applicationAttempt
+        .getAppAttemptId(), RMAppAttemptEventType.LAUNCH_FAILED));
+
+    assertEquals(RMAppAttemptState.FAILED,
+        applicationAttempt.getAppAttemptState());
+  }
+
   @Test
   public void testAMCrashAtAllocated() {
     Container amContainer = allocateApplicationAttempt();
@@ -1163,7 +1211,7 @@ public class TestRMAppAttemptTransitions {
     when(allocation.getContainers()).
         thenReturn(Collections.singletonList(amContainer));
     when(scheduler.allocate(any(ApplicationAttemptId.class), any(List.class),
-        any(List.class), any(List.class), any(List.class),
+        any(List.class), any(List.class), any(List.class), any(List.class),
         any(ContainerUpdates.class)))
         .thenReturn(allocation);
     RMContainer rmContainer = mock(RMContainerImpl.class);
@@ -1488,119 +1536,6 @@ public class TestRMAppAttemptTransitions {
         .handle(Mockito.any(RMNodeEvent.class));
   }
 
-  /**
-   * Check a completed container that is not yet pulled by AM heartbeat,
-   * is ACKed to NM for cleanup when the AM container exits.
-   */
-  @Test
-  public void testFinishedContainerNotBeingPulledByAMHeartbeat() {
-    Container amContainer = allocateApplicationAttempt();
-    launchApplicationAttempt(amContainer);
-    runApplicationAttempt(amContainer, "host", 8042, "oldtrackingurl", false);
-
-    application.handle(new RMAppRunningOnNodeEvent(application
-        .getApplicationId(), amContainer.getNodeId()));
-
-    // Complete a non-AM container
-    ContainerId containerId1 = BuilderUtils.newContainerId(applicationAttempt
-        .getAppAttemptId(), 2);
-    Container container1 = mock(Container.class);
-    ContainerStatus containerStatus1 = mock(ContainerStatus.class);
-    when(container1.getId()).thenReturn(
-        containerId1);
-    when(containerStatus1.getContainerId()).thenReturn(containerId1);
-    when(container1.getNodeId()).thenReturn(NodeId.newInstance("host", 1234));
-    applicationAttempt.handle(new RMAppAttemptContainerFinishedEvent(
-        applicationAttempt.getAppAttemptId(), containerStatus1,
-        container1.getNodeId()));
-
-    // Verify justFinishedContainers
-    ArgumentCaptor<RMNodeFinishedContainersPulledByAMEvent> captor =
-        ArgumentCaptor.forClass(RMNodeFinishedContainersPulledByAMEvent.class);
-    Assert.assertEquals(1, applicationAttempt.getJustFinishedContainers()
-        .size());
-    Assert.assertEquals(container1.getId(), applicationAttempt
-        .getJustFinishedContainers().get(0).getContainerId());
-    Assert.assertTrue(
-        getFinishedContainersSentToAM(applicationAttempt).isEmpty());
-
-    // finish AM container to emulate AM exit event
-    containerStatus1 = mock(ContainerStatus.class);
-    ContainerId amContainerId = amContainer.getId();
-    when(containerStatus1.getContainerId()).thenReturn(amContainerId);
-    applicationAttempt.handle(new RMAppAttemptContainerFinishedEvent(
-        applicationAttempt.getAppAttemptId(), containerStatus1,
-        amContainer.getNodeId()));
-
-    Mockito.verify(rmnodeEventHandler, times(2)).handle(captor.capture());
-    List<RMNodeFinishedContainersPulledByAMEvent> containerPulledEvents =
-        captor.getAllValues();
-    // Verify AM container is acked to NM via the RMNodeEvent immediately
-    Assert.assertEquals(amContainer.getId(),
-        containerPulledEvents.get(0).getContainers().get(0));
-    // Verify the non-AM container is acked to NM via the RMNodeEvent
-    Assert.assertEquals(container1.getId(),
-        containerPulledEvents.get(1).getContainers().get(0));
-    Assert.assertTrue("No container shall be added to justFinishedContainers" +
-            " as soon as AM container exits",
-        applicationAttempt.getJustFinishedContainers().isEmpty());
-    Assert.assertTrue(
-        getFinishedContainersSentToAM(applicationAttempt).isEmpty());
-  }
-
-  /**
-   * Check a completed container is ACKed to NM for cleanup after the AM
-   * container has exited.
-   */
-  @Test
-  public void testFinishedContainerAfterAMExit() {
-    Container amContainer = allocateApplicationAttempt();
-    launchApplicationAttempt(amContainer);
-    runApplicationAttempt(amContainer, "host", 8042, "oldtrackingurl", false);
-
-    // finish AM container to emulate AM exit event
-    ContainerStatus containerStatus1 = mock(ContainerStatus.class);
-    ContainerId amContainerId = amContainer.getId();
-    when(containerStatus1.getContainerId()).thenReturn(amContainerId);
-    application.handle(new RMAppRunningOnNodeEvent(application
-        .getApplicationId(),
-        amContainer.getNodeId()));
-    applicationAttempt.handle(new RMAppAttemptContainerFinishedEvent(
-        applicationAttempt.getAppAttemptId(), containerStatus1,
-        amContainer.getNodeId()));
-
-    // Verify AM container is acked to NM via the RMNodeEvent immediately
-    ArgumentCaptor<RMNodeFinishedContainersPulledByAMEvent> captor =
-        ArgumentCaptor.forClass(RMNodeFinishedContainersPulledByAMEvent.class);
-    Mockito.verify(rmnodeEventHandler).handle(captor.capture());
-    Assert.assertEquals(amContainer.getId(),
-        captor.getValue().getContainers().get(0));
-
-    // Complete a non-AM container
-    ContainerId containerId1 = BuilderUtils.newContainerId(applicationAttempt
-        .getAppAttemptId(), 2);
-    Container container1 = mock(Container.class);
-    containerStatus1 = mock(ContainerStatus.class);
-    when(container1.getId()).thenReturn(containerId1);
-    when(containerStatus1.getContainerId()).thenReturn(containerId1);
-    when(container1.getNodeId()).thenReturn(NodeId.newInstance("host", 1234));
-    applicationAttempt.handle(new RMAppAttemptContainerFinishedEvent(
-        applicationAttempt.getAppAttemptId(), containerStatus1,
-        container1.getNodeId()));
-
-    // Verify container is acked to NM via the RMNodeEvent immediately
-    captor = ArgumentCaptor.forClass(
-        RMNodeFinishedContainersPulledByAMEvent.class);
-    Mockito.verify(rmnodeEventHandler, times(2)).handle(captor.capture());
-    Assert.assertEquals(container1.getId(),
-        captor.getAllValues().get(1).getContainers().get(0));
-    Assert.assertTrue("No container shall be added to justFinishedContainers" +
-            " after AM container exited",
-        applicationAttempt.getJustFinishedContainers().isEmpty());
-    Assert.assertTrue(
-        getFinishedContainersSentToAM(applicationAttempt).isEmpty());
-  }
-
   private static List<ContainerStatus> getFinishedContainersSentToAM(
       RMAppAttempt applicationAttempt) {
     List<ContainerStatus> containers = new ArrayList<ContainerStatus>();
@@ -1713,6 +1648,34 @@ public class TestRMAppAttemptTransitions {
     assertTrue(found);
   }
 
+  @Test
+  public void testContainerRemovedBeforeAllocate() {
+    scheduleApplicationAttempt();
+
+    // Mock the allocation of AM container
+    Container container = mock(Container.class);
+    Resource resource = BuilderUtils.newResource(2048, 1);
+    when(container.getId()).thenReturn(
+        BuilderUtils.newContainerId(applicationAttempt.getAppAttemptId(), 1));
+    when(container.getResource()).thenReturn(resource);
+    Allocation allocation = mock(Allocation.class);
+    when(allocation.getContainers()).
+        thenReturn(Collections.singletonList(container));
+    when(scheduler.allocate(any(ApplicationAttemptId.class), any(List.class),
+        any(), any(), any(), any(),
+        any(ContainerUpdates.class))).
+        thenReturn(allocation);
+
+    //container removed, so return null
+    when(scheduler.getRMContainer(container.getId())).
+        thenReturn(null);
+
+    applicationAttempt.handle(
+        new RMAppAttemptEvent(applicationAttempt.getAppAttemptId(),
+            RMAppAttemptEventType.CONTAINER_ALLOCATED));
+    assertEquals(RMAppAttemptState.SCHEDULED,
+        applicationAttempt.getAppAttemptState());
+  }
 
   @SuppressWarnings("deprecation")
   @Test
@@ -1751,7 +1714,7 @@ public class TestRMAppAttemptTransitions {
   public void testScheduleTransitionReplaceAMContainerRequestWithDefaults() {
     YarnScheduler mockScheduler = mock(YarnScheduler.class);
     when(mockScheduler.allocate(any(ApplicationAttemptId.class),
-        any(List.class), any(List.class), any(List.class), any(List.class),
+        any(List.class), any(List.class), any(List.class), any(List.class), any(List.class),
         any(ContainerUpdates.class)))
         .thenAnswer(new Answer<Allocation>() {
 
